@@ -4,8 +4,8 @@
 ! '-fno-builtin-acc_on_device' is valid for C/C++/ObjC/ObjC++ but not
 ! for Fortran.
 
+      USE OPENACC
       IMPLICIT NONE
-      INCLUDE "openacc_lib.h"
 
 !Host.
 
