@@ -1,6 +1,6 @@
-// { dg-options "-std=gnu++11" }
+// { dg-do run { target c++11 } }
 
-// Copyright (C) 2005-2016 Free Software Foundation, Inc.
+// Copyright (C) 2005-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -27,8 +27,6 @@ using std::get_deleter;
 // libstdc++/24595
 void test01()
 {
-  bool test __attribute__((unused)) = true;
-
   std::shared_ptr<int> sp;
   VERIFY( !get_deleter<void(*)(int*)>(sp) );
 }
