@@ -1,6 +1,6 @@
 // 2001-06-03 pme
 
-// Copyright (C) 2001-2016 Free Software Foundation, Inc.
+// Copyright (C) 2001-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -27,7 +27,6 @@
 void 
 test01(void)
 {
-  bool test __attribute__((unused)) = true;
   const size_t n1 = 5;
 
   // the other 22 member functions should be in here too...
@@ -42,7 +41,6 @@ test01(void)
   catch(...) {
     VERIFY( false );
   }
-  VERIFY( test );
 }
 
 int main()
