@@ -1,6 +1,6 @@
 // 2004-03-02  Petur Runolfsson  <peturr02@ru.is>
 
-// Copyright (C) 2004-2016 Free Software Foundation, Inc.
+// Copyright (C) 2004-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -20,25 +20,21 @@
 // 27.4.3 fpos
 
 #include <iterator>
-#include <sstream>
+#include <limits>
 #include <testsuite_hooks.h>
 
-// libstdc++/14320
 void test01()
 {
   using namespace std;
-  bool test __attribute__((unused)) = true;
 
   typedef istreambuf_iterator<char>::difference_type Distance;
-	
-  Distance d;
-  istringstream in("5");
-  in >> d;
-  VERIFY( d == 5 );
+  typedef numeric_limits<Distance> Limits;
+
+  VERIFY( Limits::is_specialized );
+  VERIFY( Limits::is_signed );
 }
 
 int main()
 {
   test01();
-  return 0;
 }
