@@ -1,6 +1,6 @@
 // 2003-02-26 Benjamin Kosnik <bkoz@redhat.com>
 
-// Copyright (C) 2003-2016 Free Software Foundation, Inc.
+// Copyright (C) 2003-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -26,21 +26,8 @@ int main()
 {
   using namespace __gnu_test;
 
-  // ICE on figmative float template parameters?
-  /*
-  demangle("_Z3absILd1c1f1496f8a44219EEvv", "void abs<3.14159e-173>()");
-  demangle("_Z3absILd40092acd9e83e426EEvv", "void abs<3.1459>()");
-  demangl("_Z3absILe08042191a6cc56a2fe117becEEvv", "void abs<1.234e-2345l>()");
-  demangle("_Z3absILf4016147bEEvv", "void abs<2.345f>()");
-  demangle("_Z3absILfc1800000EEvv", " void abs<-16f>()");
-  demangle("_Z3absILe804bfff8000000000000000EEvv", "void abs<-1l>()");
-  */
-
-  // template<int D> void abs(void) { };
-  // template void abs<11>(void);
-  // Equivalent, but formatting difference in void argument.
-  // verify_demangle("_Z3absILi11EEvv", "void abs<(int)11>()");
-  verify_demangle("_Z3absILi11EEvv", "void abs<11>()");
+  verify_demangle("_ZlsRKU3fooU4bart1XS2_", 
+		  "operator<<(X bart foo const&, X bart foo const)");
 
   return 0;
 }
