@@ -1,7 +1,7 @@
-// { dg-options "-std=gnu++11" }
+// { dg-do run { target c++11 } }
 // { dg-require-string-conversions "" }
 
-// Copyright (C) 2013-2016 Free Software Foundation, Inc.
+// Copyright (C) 2013-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -25,10 +25,11 @@ typedef __gnu_cxx::__versa_string<char> string;
 
 void test01()
 {
-  string s1;
-  string s2 = std::move(s1);
+  string s1("string");
+  string s2("");
+  std::swap(s1, s2);
 
-  VERIFY( s2.c_str()[0] == '\0' );
+  VERIFY( s1.c_str()[0] == '\0' );
 }
 
 int main()
